//    DLC-Decrypter-Library - Java
//    Copyright (C) 2012  Julius Fischer webmaster@it-gecko.de
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.

package de.itgecko.dlc.decrypter;

import java.io.IOException;
import java.io.StringReader;
import java.util.ArrayList;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.apache.commons.codec.binary.Base64;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

/**
 * Inhalt des entschlsselten DLC Packetes
 * @author xeno
 *
 */
public class DLCContent {
	
	private ArrayList<DLCFile> dlcFiles = new ArrayList<>();
	private String generatorApp;
	private String generatorVersion;
	private String generatorUrl;
	private String dlcXmlVersion;
	private String dlcPackage;
	
	public DLCContent(String xml)
	{
		parseXML(xml);
	}

	/**
	 * @return the dlcFiles
	 */
	public ArrayList<DLCFile> getDlcFiles() {
		return dlcFiles;
	}
	
	private void parseXML(String xml)
	{
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		
		factory.setValidating(false);
		
	    DocumentBuilder builder;
		try {
			
			builder = factory.newDocumentBuilder();
			Document doc = builder.parse(new InputSource(new StringReader(xml)));
			
			NodeList cNodes = doc.getFirstChild().getChildNodes();

			for(int i = 0, c = cNodes.getLength(); i < c; i++)
			{
				Node n = cNodes.item(i);
				
				if(n.getNodeName().equalsIgnoreCase("header"))
				{
					NodeList header = n.getChildNodes();

					for(int j = 0, c2 = header.getLength(); j < c2; j++)
					{
						if(header.item(j).getNodeName().equalsIgnoreCase("generator"))
						{
							NodeList generator = header.item(j).getChildNodes();
							
							for(int k = 0, c3 = generator.getLength(); k < c3; k++)
							{
								Node generatorAttr = generator.item(k);
								
								if(generatorAttr.getNodeName().equalsIgnoreCase("app"))
									generatorApp = new String(Base64.decodeBase64(generatorAttr.getTextContent()));
								else if(generatorAttr.getNodeName().equalsIgnoreCase("version"))
									generatorVersion= new String(Base64.decodeBase64(generatorAttr.getTextContent()));
								else if(generatorAttr.getNodeName().equalsIgnoreCase("url"))
									generatorUrl = new String(Base64.decodeBase64(generatorAttr.getTextContent()));
							}
						}
						else if(header.item(j).getNodeName().equalsIgnoreCase("dlcxmlversion"))
						{
							dlcXmlVersion =  new String(Base64.decodeBase64(header.item(j).getTextContent()));
						}
					}
				}
				else if(n.getNodeName().equalsIgnoreCase("content"))
				{
					NodeList content = n.getChildNodes().item(1).getChildNodes();
					
					dlcPackage = new String(Base64.decodeBase64(n.getChildNodes().item(1).getAttributes().getNamedItem("name").getNodeValue()));
					
					for(int j = 0, c2 = content.getLength(); j < c2; j++)
					{
						if(content.item(j).getNodeName().equalsIgnoreCase("file"))
						{			
							NodeList file = content.item(j).getChildNodes();
							DLCFile dlcFile	= new DLCFile();

							for(int k = 0, c3 = file.getLength(); k < c3; k++)
							{
								Node fileNode	= file.item(k);
								
								if(fileNode.getNodeName().equalsIgnoreCase("url"))
								{
									dlcFile.setUrl(new String(Base64.decodeBase64(fileNode.getTextContent())));
								}
								else if(fileNode.getNodeName().equalsIgnoreCase("filename"))
								{
									dlcFile.setFilename(new String(Base64.decodeBase64(fileNode.getTextContent())));
									dlcFiles.add(dlcFile);
								}
							}
						}
					}
				}
			}
			
		} catch (ParserConfigurationException | SAXException | IOException e) {
			e.printStackTrace();
		}
	}

	/**
	 * @return the generatorApp
	 */
	public String getGeneratorApp() {
		return generatorApp;
	}

	/**
	 * @return the generatorVersion
	 */
	public String getGeneratorVersion() {
		return generatorVersion;
	}

	/**
	 * @return the generatorUrl
	 */
	public String getGeneratorUrl() {
		return generatorUrl;
	}

	/**
	 * @return the dlcXmlVersion
	 */
	public String getDlcXmlVersion() {
		return dlcXmlVersion;
	}

	/**
	 * @return the dlcPackage
	 */
	public String getDlcPackage() {
		return dlcPackage;
	}
}
