//    DLC-Decrypter-Library - Java
//    Copyright (C) 2012  Julius Fischer webmaster@it-gecko.de
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.

package de.itgecko.dlc.decrypter;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;
import java.security.InvalidAlgorithmParameterException;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.crypto.BadPaddingException;
import javax.crypto.Cipher;
import javax.crypto.IllegalBlockSizeException;
import javax.crypto.NoSuchPaddingException;
import javax.crypto.spec.IvParameterSpec;
import javax.crypto.spec.SecretKeySpec;
import javax.xml.bind.DatatypeConverter;

import org.apache.commons.codec.binary.Base64;

/**
* <b>DLC-Decrypter-Library</b><br><br>
* 
*  Entschsseln von DLC-Container
*   
* @author Julius Fischer
* @copyright 2012 Julius Fischer
* 
* @license GNU LGPL <br>
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version.
* 
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
* 
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
* 
* @link http:\\www.it-gecko.de
* @version 0.02
*
*/
public class DLCDecrypter {

	//private static final byte[] pKey = DatatypeConverter.parseHexBinary("9f0abe4c990992bec62c54043d08c26f");
	private static final byte[] pKey =   DatatypeConverter.parseHexBinary("2a50ca6ed2677284c3844aa6b177b715");
		
	public DLCDecrypter() {
	}
	
	/**
	 * Entschlsselt einen DLC String
	 * 
	 * @param dlc
	 * @return {@link DLCContent}
	 */
	public static DLCContent decrypt(String dlc)
	{
		return DLCDecrypter.decryptDLC(dlc);
	}
	
	/**
	 * Entschlsselt eine DLC Datei
	 * 
	 * @param file
	 * @return {@link DLCContent}
	 */
	public static DLCContent decrypt(File file)
	{
		String dlc = "";
		
		try {
			
			BufferedReader bufferedReader = new BufferedReader(new FileReader(file));
			String line;
			
			while ((line = bufferedReader.readLine()) != null) {
                dlc += line;
            }
			
			bufferedReader.close();
			
		} catch (IOException e) {
			e.printStackTrace();
		}
		
		
		return decryptDLC(dlc);
	}
	
	private static DLCContent decryptDLC(String dlc) {
		String[] rc 	= getDLCService(dlc);
		String key 	= decryptRc(rc);

		String c = decryptContent(dlc.substring(0, dlc.length() - 88).trim(), key);

		return new DLCContent(c);
		
	}

	private static String browser(String urlString, String post) {

		String content = "";

		try {
			URL url = new URL(urlString);
			URLConnection conn = url.openConnection();

			conn.setDoOutput(true);

			OutputStreamWriter out = new OutputStreamWriter(
					conn.getOutputStream());

			out.write(post);
			out.flush();

			BufferedReader in = new BufferedReader(new InputStreamReader(
					conn.getInputStream()));

			String line;
			while ((line = in.readLine()) != null) {
				content += line;
			}

			in.close();
			in.close();

		} catch (MalformedURLException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}

		return content;
	}
	
	private static String[] getDLCService(String dlc) {
		String content = browser(
				"http://service.jdownloader.org/dlcrypt/service.php",
				"destType=jdtc5&b=last09&p=2009&srcType=dlc&data="+ dlc.substring(dlc.length() - 88).trim() +"&v=9.581");
		
		if (content.isEmpty() || !content.matches("(.*)<rc>(.*)</rc>"))
			return null;

		String[] v = new String[2];
		Matcher matcher = Pattern.compile("(.*)<rc>(.*)</rc>").matcher(content);
		
		if(matcher.find())
			v[0] = matcher.group(2);
		
		matcher = Pattern.compile("bla ([0-9a-fA-F]{32})").matcher(content);
		
		if(matcher.find())
			v[1] = matcher.group(1);
		
		return v;
	}

	private static String decryptRc(String[] rc) {
		byte[] j = Base64.decodeBase64(rc[0]);
		byte[] key = pKey;
		
		if(rc[1] != null)
			key  = DatatypeConverter.parseHexBinary(rc[1]);
		
		String r = null;
		try {

			Cipher c = Cipher.getInstance("AES/ECB/NoPadding");
			c.init(Cipher.DECRYPT_MODE, new SecretKeySpec(key, "AES"));
			byte[] o = c.doFinal(j);
			r = new String(Base64.decodeBase64(o)).substring(0, 16);

		} catch (NoSuchAlgorithmException | NoSuchPaddingException
				| InvalidKeyException | IllegalBlockSizeException
				| BadPaddingException e) {
			e.printStackTrace();
		}

		return r;
	}

	private static String decryptContent(String a, String b) {
		byte[] c = Base64.decodeBase64(a);
		byte[] d = b.getBytes();
		String x = null;

		Cipher cip;
		try {

			cip = Cipher.getInstance("AES/CBC/NoPadding");
			cip.init(2, new SecretKeySpec(d, "AES"), new IvParameterSpec(d));
			x = new String(Base64.decodeBase64(cip.doFinal(c)));

		} catch (NoSuchAlgorithmException | NoSuchPaddingException
				| InvalidKeyException | InvalidAlgorithmParameterException
				| IllegalBlockSizeException | BadPaddingException e) {
			e.printStackTrace();
		}

		return x;
	}
}
